@@ -35,6 +35,7 @@ from anaml_client.models.user_group import UserGroup, UserGroupName
 # are processed.
 #
 # This is supported in Python 3.7+
+from .models.lineage import Lineage
 from .models.table_monitoring import MonitoringResultPartial, MonitoringResult
 
 if typing.TYPE_CHECKING:
@@ -62,6 +63,7 @@ from .models.destination import (
 )
 from .models.destination_reference import TableDestinationReference, FolderDestinationReference
 from .models.event_description import EventDescription, TimestampInfo
+from .models.event_store import EventStore, EventStoreId
 from .models.generated_features import GeneratedFeatures
 from .models.feature import Feature
 from .models.feature_creation_request import EventFeatureCreationRequest, FeatureCreationRequest
@@ -913,6 +915,17 @@ class Anaml:
         runs = self.get_feature_store_runs(feature_store_id=feature_store.id.value, num=1)
         return runs[0]
 
+    # EventStore-related functions
+    def get_event_stores(self) -> List[EventStore]:
+        """Get a list of all event stores from the Anaml server.
+
+        Returns:
+            A list of event stores.
+        """
+        r = self._get("/event-store")
+        result = self._json_or_handle_errors(r)
+        return [EventStore.from_json(d) for d in result]
+
     def get_merge_requests(self, limit: int = 10) -> List[MergeRequest]:
         """List merge requests from the Anaml server.
 
@@ -1085,6 +1098,16 @@ class Anaml:
         result = self._json_or_handle_errors(r)
         return Entity.from_json(result)
 
+    def get_sources(self) -> List[Source]:
+        """Get all sources from the Anaml server.
+
+        Returns:
+            All existing sources
+        """
+        r = self._get("/source")
+        result = self._json_or_handle_errors(r)
+        return [Source.from_json(d) for d in result]
+
     def get_source_by_id(self, source_id: Union[SourceId, int]) -> Source:
         """Get a source from the Anaml server.
 
@@ -1222,6 +1245,97 @@ class Anaml:
 
         return users
 
+    def get_lineage_for_event_store(self, event_store_id: Union[EventStoreId, int]) -> Lineage:
+        """Get lineage data relating to an event store from the Anaml server.
+
+        Args:
+            event_store_id: Unique identifier of the event store.
+
+        Returns:
+            The event store lineage data, if the event store exists.
+        """
+        r = self._get(f"/lineage/event-store/{int(event_store_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
+    def get_lineage_for_source(self, source_id: Union[SourceId, int]) -> Lineage:
+        """Get lineage data relating to a source from the Anaml server.
+
+        Args:
+            source_id: Unique identifier of the source.
+
+        Returns:
+            The source lineage data, if the source exists.
+        """
+        r = self._get(f"/lineage/source/{int(source_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
+    def get_lineage_for_table(self, table_id: Union[TableId, int]) -> Lineage:
+        """Get lineage data relating to a table from the Anaml server.
+
+        Args:
+            table_id: Unique identifier of the table.
+
+        Returns:
+            The table lineage data, if the table exists.
+        """
+        r = self._get(f"/lineage/table/{int(table_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
+    def get_lineage_for_feature(self, feature_id: Union[FeatureId, int]) -> Lineage:
+        """Get lineage data relating to a feature from the Anaml server.
+
+        Args:
+            feature_id: Unique identifier of the feature.
+
+        Returns:
+            The feature lineage data, if the feature exists.
+        """
+        r = self._get(f"/lineage/feature/{int(feature_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
+    def get_lineage_for_feature_set(self, feature_set_id: Union[FeatureSetId, int]) -> Lineage:
+        """Get lineage data relating to a feature set from the Anaml server.
+
+        Args:
+            feature_set_id: Unique identifier of the feature set.
+
+        Returns:
+            The feature set lineage data, if the feature set exists.
+        """
+        r = self._get(f"/lineage/feature-set/{int(feature_set_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
+    def get_lineage_for_feature_store(self, feature_store_id: Union[FeatureStoreId, int]) -> Lineage:
+        """Get lineage data relating to a feature store from the Anaml server.
+
+        Args:
+            feature_store_id: Unique identifier of the feature store.
+
+        Returns:
+            The feature store lineage data, if the feature store exists.
+        """
+        r = self._get(f"/lineage/feature-store/{int(feature_store_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
+    def get_lineage_for_destination(self, destination_id: Union[DestinationId, int]) -> Lineage:
+        """Get lineage data relating to a destination from the Anaml server.
+
+        Args:
+            destination_id: Unique identifier of the destination.
+
+        Returns:
+            The destination lineage data, if the destination exists.
+        """
+        r = self._get(f"/lineage/destination/{int(destination_id)}")
+        result = self._json_or_handle_errors(r)
+        return Lineage.from_json(result)
+
     # Builder functions
     def build_event_feature(
         self,
@@ -1608,6 +1722,17 @@ class Anaml:
     def _to_list(gotten: Optional[A]) -> List[A]:
         return [] if gotten is None else [gotten]
 
+    def _handle_errors(self, r):
+        try:
+            response_json = r.json()
+            if "errors" in response_json:
+                ex = AnamlError.from_json(response_json)
+                raise ex
+        except json.decoder.JSONDecodeError:
+            # This is not unexpected. 404 responses, in particular, often have no JSON error message.
+            pass
+        r.raise_for_status()
+
     def _json_or_handle_errors(self, r: requests.Response):
         if r.ok:
             try:
@@ -1619,24 +1744,13 @@ class Anaml:
                 self._log.error("Response content: " + r.text)
                 raise AnamlError(errors=[Reason(message="Expected JSON but the server did not return any")])
         else:
-            try:
-                response_json = r.json()
-                if "errors" in response_json:
-                    ex = AnamlError.from_json(response_json)
-                    raise ex
-            except json.decoder.JSONDecodeError:
-                # This is not unexpected. 404 responses, in particular, often have no JSON error message.
-                pass
-            r.raise_for_status()
+            self._handle_errors(r)
 
     def _int_or_handle_errors(self, r):
         if r.ok:
             return int(r.text)
         else:
-            if "errors" in r:
-                self._log.error(json.dumps(r._get("errors"), indent=4))
-        self._log.error(f"HERE: {r.text}")
-        r.raise_for_status()
+            self._handle_errors(r)
 
     def _warn_empty_feature_stats(self, features: List[str]):
         if features:
